// ----------------------------------------------------------------------------
//
//  Copyright (C) 2010-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "guiclass.h"


int Pbutt1::handle_press (void)
{
    _state ^= 3;
    return PRESS;
}

int Pbutt1::handle_relse (void)
{
    _state ^= 1;
    return NOP;
}



Rintctl::Rintctl (X_window    *parent,
                  X_callback  *cbobj,
                  int          cbind,
                  RotaryGeom  *rgeom,
                  int          xp,
                  int          yp,
		  int          cm,
                  int          dd) :
RotaryCtl (parent, cbobj, cbind, rgeom, xp, yp),
_cm (cm),
_dd (dd)
{
    _count = -1;
    set_count (0);
}

int Rintctl::handle_button (void)
{
    return PRESS;
}

int Rintctl::handle_motion (int dx, int dy)
{
    return set_count (_rcount + (dx - dy) / _dd);
}

int Rintctl::handle_mwheel (int dw)
{
    return set_count (_count + dw);
}

int Rintctl::set_count (int u)
{
    if (u <   0) u = 0;
    if (u > _cm) u = _cm;
    if (u != _count)
    {
	_count = u;
	_angle = 270.0 * ((double) u / _cm - 0.5);
        return DELTA;
    }
    return 0;
}



Rlinctl::Rlinctl (X_window    *parent,
                  X_callback  *cbobj,
                  int          cbind,
                  RotaryGeom  *rgeom,
                  int          xp,
                  int          yp,
		  int          cm,
		  int          dd,
                  double       vmin,
	          double       vmax,
	          double       vini) :
RotaryCtl (parent, cbobj, cbind, rgeom, xp, yp),
_cm (cm),
_dd (dd),
_vmin (vmin),
_vmax (vmax),
_form (0)
{
    _count = -1;
    set_value (vini);
}

void Rlinctl::get_string (char *p, int n)
{
    if (_form) snprintf (p, n, _form, _value);
    else *p = 0;
}

void Rlinctl::set_value (double v)
{
    set_count ((int) floor (_cm * (v - _vmin) / (_vmax - _vmin) + 0.5));
    render ();
}

int Rlinctl::handle_button (void)
{
    return PRESS;
}

int Rlinctl::handle_motion (int dx, int dy)
{
    return set_count (_rcount + dx - dy);
}

int Rlinctl::handle_mwheel (int dw)
{
    if (! (_keymod & ShiftMask)) dw *= _dd;
    return set_count (_count + dw);
}

int Rlinctl::set_count (int u)
{
    if (u <   0) u = 0;
    if (u > _cm) u = _cm;
    if (u != _count)
    {
	_count = u;
	_value = _vmin + u * (_vmax - _vmin) / _cm;
	_angle = 270.0 * ((double) u / _cm - 0.5);
        return DELTA;
    }
    return 0;
}



Rlogctl::Rlogctl (X_window    *parent,
                  X_callback  *cbobj,
                  int          cbind,
                  RotaryGeom  *rgeom,
                  int          xp,
                  int          yp,
		  int          cm,
		  int          dd,
                  double       vmin,
	          double       vmax,
	          double       vini) :
RotaryCtl (parent, cbobj, cbind, rgeom, xp, yp),
_cm (cm),
_dd (dd),
_form (0)
{
    _count = -1;
    _vmin = log (vmin);
    _vmax = log (vmax);
    set_value (vini);
}

void Rlogctl::get_string (char *p, int n)
{
    if (_form) snprintf (p, n, _form, _value);
    else *p = 0;
}

void Rlogctl::set_value (double v)
{
    set_count ((int) floor (_cm * (log (v) - _vmin) / (_vmax - _vmin) + 0.5));
    render ();
}

int Rlogctl::handle_button (void)
{
    return PRESS;
}

int Rlogctl::handle_motion (int dx, int dy)
{
    return set_count (_rcount + dx - dy);
}

int Rlogctl::handle_mwheel (int dw)
{
    if (! (_keymod & ShiftMask)) dw *= _dd;
    return set_count (_count + dw);
}

int Rlogctl::set_count (int u)
{
    if (u <   0) u = 0;
    if (u > _cm) u = _cm;
    if (u != _count)
    {
	_count = u;
	_value = exp (_vmin + u * (_vmax - _vmin) / _cm);
	_angle = 270.0 * ((double) u / _cm - 0.5);
        return DELTA;
    }
    return 0;
}


Rgainctl::Rgainctl (X_window    *parent,
                    X_callback  *cbobj,
                    int          cbind,
                    RotaryGeom  *rgeom,
                    int          xp,
                    int          yp,
                    int          maxdb) :
RotaryCtl (parent, cbobj, cbind, rgeom, xp, yp),
_maxdb (maxdb),    
_lastc (0)
{
    _count = -1;
    set_count (0);
}


void Rgainctl::get_string (char *p, int n)
{
    snprintf (p, n, "%5.1lf", _value);
}


void Rgainctl::set_value (double v)
{
    double u;

    v -= _maxdb;
    if       (v >= -40) u = 300 + 5 * v;
    else  if (v > -100) u = (1 - sqrt (1 - 16e-3 * (100 + v))) / 8e-3;
    else                u = 0;
    set_count ((int) floor (u + 0.5));
    render ();
}


int Rgainctl::handle_button (void)
{
    int k;

    if (_button == 3)
    {
	if (_count) { k = 0; _lastc = _count; }
	else   	    { k = _lastc; _lastc = 0; }
        return set_count (k);
    }
    return PRESS;
}


int Rgainctl::handle_motion (int dx, int dy)
{
    return set_count (_rcount + 2 * (dx - dy));
}


int Rgainctl::handle_mwheel (int dw)
{
    if (! (_keymod & ShiftMask)) dw *= 5;
    return set_count (_count + dw);
}


int Rgainctl::set_count (int u)
{
    if (u <   0) u = 0;
    if (u > 300) u = 300;
    if (u != _count)
    {
	_count = u;
	if      (u > 100) _value = 0.2 * (u - 300) + _maxdb;
        else if (u)       _value = (1 - 4e-3 * u) * u - 100 + _maxdb;
	else              _value = -1000;
	_angle = 0.9 * (u - 150);
        return DELTA;
    }
    return 0;
}


