// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <jack/midiport.h>
#include "jclient.h"
#include "global.h"


Jclient::Jclient (const char *jname, const char *jserv) :
    A_thread ("jclient"),
    _jack_client (0),
    _midi_chan (-1),
    _midi_note (0),
    _active (false),
    _jname (0),
    _vocoder (0)
{
    init_jack (jname, jserv);  
}


Jclient::~Jclient (void)
{
    if (_jack_client) close_jack ();
}


void Jclient::init_jack (const char *jname, const char *jserv)
{
    jack_status_t  stat;
    int            opts;

    opts = JackNoStartServer;
    if (jserv) opts |= JackServerName;
    if ((_jack_client = jack_client_open (jname, (jack_options_t) opts, &stat, jserv)) == 0)
    {
        fprintf (stderr, "Can't connect to JACK.\n");
        exit (1);
    }
    jack_on_shutdown (_jack_client, jack_static_shutdown, (void *) this);
    jack_set_process_callback (_jack_client, jack_static_process, (void *) this);
    if (jack_activate (_jack_client))
    {
        fprintf(stderr, "Can't activate JACK.\n");
        exit (1);
    }

    _jname = jack_get_client_name (_jack_client);
    _fsamp = jack_get_sample_rate (_jack_client);
    _fsize = jack_get_buffer_size (_jack_client);

    _ainp_port [0] = jack_port_register (_jack_client, "voice_in",  JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput,  0);
    _ainp_port [1] = jack_port_register (_jack_client, "excit_vo",  JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput,  0);
    _ainp_port [2] = jack_port_register (_jack_client, "excit_uv",  JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput,  0);
    _aout_port [0] = jack_port_register (_jack_client, "out", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
    _midi_port = jack_port_register (_jack_client, "pitch", JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
	
    _vocoder = new Vocoder (_fsamp);
    _active = true;
}


void Jclient::close_jack ()
{
    jack_deactivate (_jack_client);
    jack_client_close (_jack_client);
    delete _vocoder;
}


void Jclient::jack_static_shutdown (void *arg)
{
    ((Jclient *) arg)->jack_shutdown ();    
}


int Jclient::jack_static_process (jack_nframes_t nframes, void *arg)
{
    return ((Jclient *) arg)->jack_process (nframes);
}


void Jclient::jack_shutdown (void)
{
    send_event (EV_EXIT, 1);
}




int Jclient::jack_process (int nframes)
{
    int   i;
    float *inpp [3];
    float *outp [1];

    if (!_active) return 0;

    midi_process (nframes);
    for (i = 0; i < 3; i++)  inpp [i] = (float *) jack_port_get_buffer (_ainp_port [i], nframes);
    for (i = 0; i < 1; i++)  outp [i] = (float *) jack_port_get_buffer (_aout_port [i], nframes);
    _vocoder->process (nframes, inpp, outp);
 
    return 0;
}


void Jclient::midi_process (int nframes)
{
    int                i, n, t, v;
    void               *p;
    jack_midi_event_t  E;

    p = jack_port_get_buffer (_midi_port, nframes);
    i = 0;
    while (jack_midi_event_get (&E, p, i) == 0)
    {
	t = E.buffer [0];
	if ((_midi_chan < 0) || ((t & 0x0F) == _midi_chan))
	{
            n = E.buffer [1];
	    if ((n >= 24) && (n <= 84))
	    {
	        v = E.buffer [2];
	        if ((t == 0x90 ) && v)
		{
		    _midi_note = n;
		    _vocoder->set_midinote (n, true);
		}
		if ((t == 0x80) || !v)
		{
		    if (n == _midi_note)
		    {
		        _vocoder->set_midinote (n, false);
		    }
		}
  	    }
	}
	i++;
    }
}


