// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "styles.h"
#include "global.h"
#include "mainwin.h"


Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, int xp, int yp, Jclient *jclient) :
    A_thread ("Main"),
    X_window (parent, xp, yp, XSIZE, YSIZE, XftColors [C_MAIN_BG]->pixel),
    _stop (false),
    _xres (xres),
    _jclient (jclient)
{
    X_hints  H;
    char     s [256];

    _atom = XInternAtom (dpy (), "WM_DELETE_WINDOW", True);
    XSetWMProtocols (dpy (), win (), &_atom, 1);
    _atom = XInternAtom (dpy (), "WM_PROTOCOLS", True);

    sprintf (s, "%s", jclient->jname ());
    x_set_title (s);
    H.position (xp, yp);
    H.minsize (XSIZE, YSIZE);
    H.maxsize (XSIZE, YSIZE);
    H.rname (xres->rname ());
    H.rclas (xres->rclas ());
    x_apply (&H); 

    RotaryCtl::init (disp ());
    
    _vocoder = _jclient->vocoder ();

    _oscilwin = new Oscilwin (this,  51,  17, _vocoder);
    _excwinV  = new Excitwin (this, 311,  17, _vocoder, 'V');
    _excwinU  = new Excitwin (this, 521,  17, _vocoder, 'U');
    _inputwin = new Inputwin (this,   0,  95, _vocoder);
    _filtwin  = new Filtwin  (this, 195,  75, _vocoder);
    
    _gmidi._backg = XftColors [C_MAIN_BG];
    _gmidi._image [0] = midichan_img;
    _gmidi._lncol [0] = 1;
    _gmidi._x0 = 7;
    _gmidi._y0 = 2;
    _gmidi._dx = 35;
    _gmidi._dy = 35;
    _gmidi._xref = 17.5;
    _gmidi._yref = 17.5;
    _gmidi._rad = 11;

    bstyle1.size.x = 50;
    bstyle1.size.y = 17;
    _tmidi = new X_tbutton (this, 0, &bstyle1, 0, 54, "Omni", 0, 0);
    _tmidi->x_map ();
    _rmidi = new Rintctl (this, this, R_MIDI, &_gmidi, 0, 17, 16, 15);
    _rmidi->x_map ();
    
    (new X_textln (this, &tstyle1,   6,  0,  44, 17, "MIDI", 0))->x_map ();
    (new X_textln (this, &tstyle1,  56,  0, 100, 17, "Oscillator", -1))->x_map ();
    (new X_textln (this, &tstyle1, 316,  0, 100, 17, "V excitation", -1))->x_map ();
    (new X_textln (this, &tstyle1, 526,  0, 100, 17, "U excitation", -1))->x_map ();
    (new X_textln (this, &tstyle1,   5, 75, 130, 17, "Voice analyser", -1))->x_map ();
    
    x_add_events (ExposureMask); 
    x_map (); 
    set_time (0);
    inc_time (100000);
}

 
Mainwin::~Mainwin (void)
{
    RotaryCtl::fini ();
}

 
int Mainwin::process (void)
{
    int e;

    if (_stop) handle_stop ();

    e = get_event_timed ();
    switch (e)
    {
    case EV_TIME:
        handle_time ();
	break;
    }
    return e;
}


void Mainwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;  
 
    case ClientMessage:
        clmesg ((XClientMessageEvent *) E);
        break;
    }
}


void Mainwin::expose (XExposeEvent *E)
{
    if (E->count) return;
    redraw ();
}


void Mainwin::clmesg (XClientMessageEvent *E)
{
    if (E->message_type == _atom) _stop = true;
}


void Mainwin::handle_time (void)
{
    _filtwin->update ();
    _inputwin->update ();
    inc_time (25000);
    XFlush (dpy ());
}


void Mainwin::handle_stop (void)
{
    put_event (EV_EXIT, 1);
}


void Mainwin::handle_callb (int type, X_window *W, XEvent *E)
{
    RotaryCtl *R;
    int       k;
    char      s [8];
    
    switch (type)
    {
    case RotaryCtl::DELTA:
	R = (RotaryCtl *) W;
        switch (R->cbind ())
	{
	case R_MIDI:
	    k = _rmidi->count ();
	    if (k)
	    {
		sprintf (s, "Ch %d", k);
		_tmidi->set_text (s, 0);
	    }
	    else _tmidi->set_text ("Omni", 0);
   	    _tmidi->redraw ();
	    _jclient->set_midichan (k - 1);
	    break;
	}
	break;
    }
}



void Mainwin::redraw (void)
{
    X_draw D (dpy (), win (), dgc (), xft ());

    XPutImage (dpy (), win (), dgc (), midichan_img, 0, 0, 0, 17, 49, 39);
    D.setline (1);
    D.setcolor (XftColors [C_MAIN_L2]->pixel);
    D.drawline (  0, 74, XSIZE, 74);
    D.drawline ( 50,  0,  50,   74);
    D.drawline (310,  0, 310,   74);
    D.drawline (520,  0, 520,   74);
    D.drawline (194, 74, 194, YSIZE);
}


