// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "sawosc.h"


const float F_PI = M_PI;


// Simple anti-aliased onential sawtooth oscillator.


Sawosc::Sawosc (void)
{
    int     i, n;
    float   v, a = 0.7f;

    // Windowed sinc function for anti-aliasing the step.
    // This sets a limit to the maximum frequency unless
    // process() provides for overlapping steps. The one
    // below does not (which is OK for this application).

    n = LSINC * KUPS;
    _sincp = new float [n];
    for (i = 0; i < n; i++)
    {
	v = 0.9f * F_PI * ((float) i / KUPS - LSINC / 2);
	if (fabsf (v) < 1e-6f) _sincp [i] = a;
	else _sincp [i] = a * sinf (i * F_PI / n) * sinf (v) / v;
    }
    reset ();
}


Sawosc::~Sawosc (void)
{
    delete[] _sincp;
}


void Sawosc::reset (void)
{
    _k = 0;
    _f1 = 0.01;
    _f = 0.01;
    _p = _u = _v = 0.0;
}


void Sawosc::set_freq (float freq, bool step)
{
    if (freq > 0.05f) freq = 0.05f;
    _f1 = freq;
    if (step)
    {
	_f = _f1;
        _p = 0.0;
    }
}


void Sawosc::process (int nfram, float *out)
{
    int    k;
    float  f, p, u, v, w1, w2;

    k = _k;
    f = _f;
    p = _p;
    v = _v;
    u = _u;
    w1 = 0.1f;
    w2 = 0.1 * _f1;  // Lowpass for DC removal

    while (nfram--)
    {
	f += w1 * (_f1 - f) + 1e-20f;
	if (k < LSINC * KUPS)
	{
	    v += _sincp [k];
	    k += KUPS;
	}
	v *= 1.0 - 15 * f;
        u += w2 * f * (v - u);
	*out++ = v - u;
	p += f;
	if (p >= 1.0)
	{
	    p -= 1.0;
	    k = (int)(KUPS * p / f + 0.5);
	}
    }
    
    _k = k;
    _f = f;
    _p = p;	
    _v = v;
    _u = u;
}

