// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <math.h>
#include "svfilt.h"


static const float F_PI = M_PI; 
#define EPS 1e-30f;


SVfilt::SVfilt (void) :
    _c1 (0),
    _c2 (0)
{
    reset ();
}


SVfilt::~SVfilt (void)
{
}


void SVfilt::init (float f, float q, float g)
{
    float w, a, b;
    
    w = 2 * tanf (F_PI * f);
    a = w / q;
    b = w * w;
    _c1 = a + b;
    _c2 = b / _c1;
    _c1 /= 1 + a / 2 + b / 4;
    _g = g;
}


void SVfilt::proc_lowpass (int n, float *inp, float *out)
{
    int   i;
    float x, z1, z2, a0;

    a0 = _g * _c1 * _c2 / 4;
    z1 = _z1;
    z2 = _z2;
    for (i = 0; i < n; i++)
    {
	x = inp [i] - z1 - z2 + EPS;
	z2 += _c2 * z1;
	z1 += _c1 * x;
        out [i] = a0 * x + z2;
    }
    _z1 = z1;
    _z2 = z2;
}


void SVfilt::proc_bandpass (int n, float *inp, float *out)
{
    int   i;
    float x, z1, z2, a0, a1;

    a1 = _g * (1 - _c2);
    a0 = a1 * _c1 / 2;
    z1 = _z1;
    z2 = _z2;
    for (i = 0; i < n; i++)
    {
	x = inp [i] - z1 - z2 + EPS;
        out [i] = a0 * x + a1 * z1;
	z2 += _c2 * z1;
	z1 += _c1 * x;
    }
    _z1 = z1;
    _z2 = z2;
}


void SVfilt::proc_highpass (int n, float *inp, float *out)
{
    int   i;
    float x, z1, z2, a0;

    a0 = _g * (1 - _c1 / 2 + _c1 * _c2 / 4);
    z1 = _z1;
    z2 = _z2;
    for (i = 0; i < n; i++)
    {
	x = inp [i] - z1 - z2 + EPS;
        out [i] = a0 * x;
	z2 += _c2 * z1;
	z1 += _c1 * x;
    }
    _z1 = z1;
    _z2 = z2;
}

