// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "vocoder.h"


static const float F_2PI = 2 * M_PI;
    

Vocoder::Vocoder (int fsam) :
    _fsam (fsam),
    _ifrag (0),
    _ivux (0),
    _vcd0 (false),
    _vcd1 (false),
    _vcdf (0),
    _analyser (0),
    _resynth (0),
    _rngen (0),
    _oscfreq (60),
    _osctune (0.0f),
    _oscfmod (0.0f),
    _oscgain (0.0f),
    _flfo (4.0f),
    _plfo (0.0f),
    _midi2osc (false),
    _midinote (60),
    _midigate (false),
    _Vintern (false),
    _Uintern (false),
    _hold (false),
    _Again (1.0f),
    _Vgain (1.0f),
    _Ugain (1.0f),
    _UVthr (0.0f),
    _gfade (0),
    _inpbuf (0),
    _outbuf (0),
    _Vexcit (0),
    _Uexcit (0)
{
    if      (_fsam > 128000) _frag = 128;
    else if (_fsam >  64000) _frag = 64;
    else                     _frag = 32;

    _wgosc = expf (-1.0f / (0.02f * _fsam));
    _w1nse = 6.28f * 300.0f / fsam;
    _z1nse = 0;

    _analyser = new VocAnalyser (_frag, fsam);
    _resynth  = new VocResynth (_frag, fsam);

    _xlen = 6 * _frag;
    _gfade = new float [_xlen];
    for (int i = 0; i <= _xlen; i++)
    {
        _gfade [i] = sinf (0.5 * M_PI * i / _xlen);
    }
    _inpbuf = new float [_frag];
    _outbuf = new float [_frag];
    _Vexcit = new float [_frag];
    _Uexcit = new float [_frag];
    memset (_outbuf, 0, _frag * sizeof (float));
    memset (_pband, 0, NBAND * sizeof (float));
    memset (_gband, 0, NBAND * sizeof (float));
}


Vocoder::~Vocoder (void)
{
    delete _analyser;
    delete _resynth;
    delete[] _gfade;
    delete[] _inpbuf;
    delete[] _outbuf;
    delete[] _Vexcit;
    delete[] _Uexcit;
}


float Vocoder::get_pband (int j, float m)
{
    float p;

    p = _pband [j];
    if (!_hold) _pband [j] = m * p + 1e-20f;
    return p;
}


void Vocoder::process (int nfram, float *inp[], float *out[])
{
    int n;

    while (nfram)
    {
        n = _frag - _ifrag;
        if (n > nfram) n = nfram;
        memcpy (_inpbuf + _ifrag, inp [INPA], n * sizeof (float));
        memcpy (_Vexcit + _ifrag, inp [INPV], n * sizeof (float));
        memcpy (_Uexcit + _ifrag, inp [INPU], n * sizeof (float));
        memcpy (out [0], _outbuf + _ifrag, n * sizeof (float));
        inp [INPA] += n;
        inp [INPV] += n;
        inp [INPU] += n;
        out [0] += n;
        _ifrag += n;
        if (_ifrag == _frag)
        {
            procfrag ();
            _ifrag = 0;
        }
        nfram -= n;
    }   
}


void Vocoder::procfrag (void)
{
    int     i, j, u, v;
    float   m, gv, gu;
 
    for (i = 0; i < _frag; i++) _inpbuf [i] *= _Again;
    _analyser->process (_inpbuf);

    if      (_UVthr < -17.5f) _vcd1 = false;
    else if (_UVthr >  17.5f) _vcd1 = true;
    else _vcd1 = _analyser->ratio () > 12.0f - _UVthr;
    if (!_hold) _vcdf += 0.05f * ((_vcd1 ? 1 : -1) - _vcdf);
    
    if (_Vintern) genvoiced ();
    _Veq1.process (_frag, _Vexcit, _Vexcit);
    _Veq2.process (_frag, _Vexcit, _Vexcit);

    if (_Uintern) gennoise ();
    _Ueq1.process (_frag, _Uexcit, _Uexcit);
    _Ueq2.process (_frag, _Uexcit, _Uexcit);

    v = _ivux;
    u = _xlen - v;
    for (i = 0; i < _frag; i++)
    {
        gv = _gfade [v] * _Vgain;
        gu = _gfade [u] * _Ugain;
        _inpbuf [i] = gv * _Vexcit [i] + gu * _Uexcit [i];
        if (!_hold)
        {
            if (_vcd1) { if (u) { u--; v++; }}
            else       { if (v) { v--; u++; }}
        }
    }
    _ivux = v;
    _vcd0 = _vcd1;

    if (!_hold)
    {
        for (j = 0; j < NBAND; j++)
        {
            m = _analyser->power (j);
            _resynth->set_gain (j, _gband [j] * sqrtf (m));
            if (_pband [j] < m) _pband [j] = m;
        }
    }
    _resynth->process (_inpbuf, _outbuf);
}


void Vocoder::gennoise (void)
{
    int    i;
    float  v, z;

    z = _z1nse;
    for (i = 0; i < _frag; i++)
    {
        v = _rngen.grandf ();
        z += _w1nse * (v - z);
        _Uexcit [i] = 0.2f * (v - z);
    }
    _z1nse = z;
}


void Vocoder::genvoiced (void)
{
    int    i;
    float  g, d, f, m;

    m = _midi2osc ? _midinote : _oscfreq;
    m += _osctune - 60; 
    m += _oscfmod * sinf (F_2PI * _plfo);
    _plfo += _flfo * _frag / _fsam;
    if (_plfo > 1.0f) _plfo -= 1.0f;
    f = 261.63f * powf (2, m / 12.0f);
    _oscil.set_freq (f / _fsam, 0);
    _oscil.process (_frag, _Vexcit);
    if (_midi2osc)
    {
        g = _oscgain;
	if (_midigate)
	{
	    d = (1.0f - g) / _frag;
	    for (i = 0; i < _frag; i++)
	    {
		g += d;
		_Vexcit [i] *= g;
	    }
	    _oscgain = 1.0f;
	}
	else
	{
	    if (g < 1e-3f)
	    {
		memset (_Vexcit, 0, _frag * sizeof (float));
		_oscgain = 0.0f;
	    }
	    else
	    {
	        for (i = 0; i < _frag; i++)
	        {
		    g *= _wgosc;
		    _Vexcit [i] *= g;
		}
		_oscgain = g;
	    }
	}		
    }
}

