// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#ifndef __VOCODER_H
#define __VOCODER_H


#include <math.h>
#include "filtbank3.h"
#include "sawosc.h"
#include "rngen.h"
#include "pareq.h"
#include "svfilt.h"


#define EQ1F 8e2f
#define EQ1B 2.0f
#define EQ2F 6e3f
#define EQ2B 2.5f


class Vocoder
{
public:

    Vocoder (int fsamp);
    ~Vocoder (void);

    enum { NBAND = VocAnalyser::NFILT, INPA = 0, INPV = 1, INPU = 2 };

    void  process (int nfram, float *inp[], float *out[]);

    void  set_oscfreq (int v)   { _oscfreq = v; }
    void  set_osctune (float v) { _osctune = v; }
    void  set_oscfmod (float v) { _oscfmod = v; }
    void  set_lfofreq (float v) { _flfo = v; }
    void  set_midi2osc (bool v) { _midi2osc = v; }
    void  set_midinote (int n, int v)  { _midinote = n; _midigate = v; }
    void  set_Again (float v) { _Again = (v < -60) ? 0.0f : powf (10.0f, v / 20); }
    void  set_Vgain (float v) { _Vgain = (v < -60) ? 0.0f : powf (10.0f, v / 20); }
    void  set_Vintern (bool k) { _Vintern = k; }
    void  set_Veq1g (float g) { _Veq1.set_param (EQ1F / _fsam, EQ1B, g); }
    void  set_Veq2g (float g) { _Veq2.set_param (EQ2F / _fsam, EQ2B, g); }
    void  set_Ugain (float v) { _Ugain = (v < -60) ? 0.0f : powf (10.0f, v / 20); }
    void  set_Uintern (bool k) { _Uintern = k; }
    void  set_Ueq1g (float g) { _Ueq1.set_param (EQ1F / _fsam, EQ1B, g); }
    void  set_Ueq2g (float g) { _Ueq2.set_param (EQ2F / _fsam, EQ2B, g); }
    void  set_UVthr (float v)  { _UVthr = v; }
    void  set_hold (bool k)  { _hold = k; }
    void  set_gband (int j, float g) { _gband [j] = g; }
    float get_pband (int j, float m);
    float get_vcdf (void) { return _vcdf; }
    
private:

    void procfrag (void);
    void gennoise (void);
    void genvoiced (void);
    
    int              _fsam;
    int              _frag;
    int              _xlen;
    int              _ifrag;
    int              _ivux;
    bool             _vcd0;
    bool             _vcd1;
    float            _vcdf;
    VocAnalyser     *_analyser;
    VocResynth      *_resynth;
    float            _pband [NBAND];
    float            _gband [NBAND];  
    Rngen            _rngen;
    Sawosc           _oscil;
    int              _oscfreq;
    float            _osctune;
    float            _oscfmod;
    float            _oscgain;
    float            _flfo;
    float            _plfo;
    bool             _midi2osc;
    int              _midinote;
    bool             _midigate;
    bool             _Vintern;
    bool             _Uintern;
    bool             _hold;
    float            _Again;
    float            _Vgain;
    float            _Ugain;
    float            _gvexc;
    Pareq            _Veq1;
    Pareq            _Veq2;
    Pareq            _Ueq1;
    Pareq            _Ueq2;
    float            _UVthr;
    float            _wgosc;
    float            _w1nse;
    float            _z1nse;
    float           *_gfade;
    float           *_inpbuf;
    float           *_outbuf;
    float           *_Vexcit;
    float           *_Uexcit;
};


#endif
